attribute	highp vec4	vertex_pos;
attribute	highp vec3	vertex_normal;
attribute	highp vec2	vertex_uv0;

uniform		highp	mat4	pvwMat;
uniform		highp	mat3	worldMat;
uniform		highp	vec3	eyePosMS;
uniform		mediump vec3	lightDirMS;
uniform		lowp	vec3	lightAmbient;

uniform		highp	vec3	pointLight[4];
uniform             int		numPointLights;

varying		highp   vec2	uv0Var;
varying		highp   vec2	uv1Var;
varying		mediump vec3	halfVecMSVar;
varying		lowp	float	reflDownAmmount;
varying		lowp	vec3	pointDiffuse;
varying		lowp	float	pointSpecular;

const highp float OneOverPi = 0.318309886183791;
const highp vec3 Goldenrod = vec3(1.0, 0.756880733944954, 0.146788990825688);

void main()
{
	uv0Var		= vertex_uv0;
	
	highp vec3 eyeDirMS = normalize(eyePosMS - vertex_pos.xyz);
	halfVecMSVar = eyeDirMS + lightDirMS;
	highp vec3 reflDirMS = worldMat * reflect(-eyeDirMS, vertex_normal);
	
	highp float d = clamp(acos(reflDirMS.y) * OneOverPi * 1.3, 0.0, 1.0);
	highp float a = atan(-reflDirMS.x, -reflDirMS.z);

	uv1Var.x = cos(a) * d * 0.5 + 0.5;
	uv1Var.y = sin(a) * d * 0.5 + 0.5;
	
	reflDownAmmount = clamp(100.0 * (1.0 - clamp((0.6 + reflDirMS.y) / 0.4 + 1.0, 0.0, 1.0)), 0.0, 1.0);
	
	highp float diffuse = 0.0;
	highp float specular = 0.0;
	highp vec3	dir;
	highp float dist;
	
	for(int i = 0; i < numPointLights; ++i)
	{
		dir = pointLight[i] - vertex_pos.xyz;
		dist = length(dir);
		dir /= dist;
		
		dist = max((20.0 - dist) * 0.05, 0.0);
		dir.x = dot(vertex_normal, dir);
		dir.y = clamp(0.5 + dir.x, 0.0, 1.0);
		
		diffuse += dir.y * dist;
		specular += max((dir.x - 0.8) * 2.5 * dist, 0.0);
	}
	
	pointDiffuse = Goldenrod * min(diffuse, 1.0) + lightAmbient;
	pointSpecular = min(pow(min(2.0 * specular, 1.0), 3.0), 0.5);
	
	gl_Position = pvwMat * vertex_pos;
}
