uniform		sampler2D	tex0; //bump0 texture
uniform		sampler2D	tex1; //bump1 texture
uniform     sampler2D   tex2; //color texture
uniform     sampler2D   tex3; //reflection texture
uniform     sampler2D   tex4; //refraction texture

uniform		mediump	vec3	sunDir;
uniform		mediump	vec3	normalBump;
uniform     highp   vec4    screenOffset;

varying		highp	vec2	tex0Var;
varying		highp	vec2	tex1Var;
varying		highp	vec3	eyeDirMSVar;
varying     highp   vec3    screenInfo;
varying		highp	vec2    colorVar;

const mediump float	specPow		= 48.0;
const mediump float kReflBump   = 0.04;
const mediump float kRefrBump   = 0.016;

void main()
{
	mediump	vec3  normal	= normalize(texture2D(tex0, tex0Var).xyz + texture2D(tex1, tex1Var).xyz - 1.0);
    mediump vec3  color     = texture2D(tex2, colorVar).rgb;
    highp vec2 scr          = screenInfo.xy / screenInfo.z + screenOffset.xy;
	mediump vec3  refl      = texture2D(tex3, normal.xz * kReflBump + scr).rgb;
    mediump vec4  refr      = texture2D(tex4, normal.xz * kRefrBump + scr);
    
    mediump	vec3  eye		= normalize(eyeDirMSVar);
	mediump vec3  halfVec	= normalize(eye + sunDir);
    mediump vec2  sPfR      = vec2(dot(normal, halfVec), dot(normal * normalBump, eye));
    
    sPfR.x = sPfR.x * specPow - specPow;
    sPfR   = exp(sPfR);
    sPfR.x = normal.x * normal.x + sPfR.x;
    sPfR.x *= 1.0 - refr.a;
    color += refl * sPfR.y + sPfR.x;
    
    gl_FragColor = vec4(mix(color, refr.rgb, refr.a), 200.0 * sPfR.x - 180.0);
}
