attribute	vec4	vertex_pos;
uniform		mat4	pvwMat;

uniform highp vec3 lightDirMS;
uniform float   fCameraHeight;
uniform float   fDepthCamera2;

uniform vec3    v3InvWavelength4KrKm4Pi;
uniform vec3    v3InvWavelength4KrESunExp;

const float fPi             = 3.141592653589793;
const float fESun           = 20.0;
const float fKr             = 0.0025;
const float fKm             = 0.0015;
const int   nSamples        = 6;
const float fSamplesInv     = 1.0 / 6.0;
const float fInnerRadius    = 10.0;
const float fOuterRadius    = 10.25;
const float fScaleDepth     = 0.25;
const float fExp            = -2.0;
const float fKr4Pi                  = fKr*4.0*fPi;
const float fKm4Pi                  = fKm*4.0*fPi;
const float fKrESunExp              = fKr*fESun*fExp;
const float fKmESunExp              = fKm*fESun*fExp;
const float fScale                  = 1.0 / (fOuterRadius - fInnerRadius);
const float fScaleOverScaleDepth    = fScale / fScaleDepth;

varying highp   vec3 uv0Var;
varying mediump vec3 color0;
varying mediump vec4 colorClouds;

float scale(float fCos)
{
	float x = 1.0 - fCos;
	return fScaleDepth * exp(x*(x*(x*(x*5.25-6.8)+3.83)+0.459)-0.00287);
}

void main(void)
{
	vec3 v3Ray = vertex_pos.xyz; v3Ray.y -= fCameraHeight;
	float fFar = length(v3Ray);
	v3Ray /= fFar;
    
	float fStartOffset = fDepthCamera2 * scale(v3Ray.y);
	float fSampleLength = fFar * fSamplesInv;
	float fScaledLength = fSampleLength * fScale;
	vec3 v3SampleRay   = v3Ray * fSampleLength;
	vec3 v3SamplePoint = v3SampleRay * 0.5; v3SamplePoint.y += fCameraHeight;
    
	vec3 v3FrontColor = vec3(0.0);
	for(int i=0; i<nSamples; i++)
	{
		float fHeight       = length(v3SamplePoint);
		float fDepth        = exp(fScaleOverScaleDepth * (fInnerRadius - fHeight));
		float fLightAngle   = max(-0.07, dot(lightDirMS, v3SamplePoint) / fHeight);
		float fCameraAngle  = dot(v3Ray, v3SamplePoint) / fHeight;
		float fScatter      = fStartOffset + fDepth * (scale(fLightAngle) - scale(fCameraAngle));
		vec3 v3Attenuate    = exp(-fScatter * v3InvWavelength4KrKm4Pi);
		v3FrontColor += v3Attenuate * (fDepth * fScaledLength);
		v3SamplePoint += v3SampleRay;
	}

    float fPhase = dot(lightDirMS, v3Ray);
    fPhase = fPhase * fPhase * 0.75 + 0.75;
    
    color0 = 1.0 - exp(v3FrontColor * (v3InvWavelength4KrESunExp * fPhase + fKmESunExp));
    colorClouds.rgb = v3FrontColor * fKmESunExp * -100.0;
    colorClouds.a = min(dot(max(1.0 - colorClouds.rgb, 0.0), vec3(0.299, 0.587, 0.114)) * 6.0, 1.0);
    
    uv0Var.xz = vertex_pos.xz;
    uv0Var.y  = vertex_pos.y * 3.0 - 30.0;
	gl_Position = pvwMat * vertex_pos;
    gl_Position.z = gl_Position.w;
}
