attribute	highp	vec4	vertex_pos;
attribute	highp   vec2	vertex_uv0;

uniform		highp mat4	pvwMat;
uniform		highp vec3	eyePosMS;

uniform		highp vec4  sunColorIntensity;
uniform		highp vec3	sunDir;

uniform		highp vec3  vBetaDashRayleigh;
uniform		highp vec3  vBetaDashMie;
uniform		highp vec3  vBetaRayleighMie;
uniform		highp vec3	vHG;

uniform		highp float fogStart;
uniform		highp float fogEnd;
const		highp float	scatterMaxDist = 100000.0;

varying		highp    vec2	uv0Var;
varying		mediump  vec3	extinction;
varying		mediump  vec3	inscatter;
varying		mediump  vec4	density;

void main()
{
	highp vec3  eyeDirMS = vertex_pos.xyz - eyePosMS;
	highp float eyeDist = clamp(length(eyeDirMS) * fogEnd - fogStart, 0.0, scatterMaxDist);
	eyeDirMS = normalize(eyeDirMS);
	
	highp float theta = max(dot(sunDir, eyeDirMS), 0.0);
	highp float phase = (theta * theta * theta + 1.0) * (theta * -0.2 + 1.0);
	highp float phaseThetaMie = vHG.z * -theta + vHG.y;
	phaseThetaMie = vHG.x * inversesqrt(phaseThetaMie) / phaseThetaMie;
	
	highp vec3 hp_inscatter = vBetaDashRayleigh * phase + vBetaDashMie * phaseThetaMie;
    hp_inscatter *= sunColorIntensity.rgb * sunColorIntensity.w;
    
    highp vec3 vRM = exp(vBetaRayleighMie * -eyeDist);
    hp_inscatter = hp_inscatter * -vRM + hp_inscatter;
    
	highp float densityH = 1.0 - clamp(vertex_pos.y * 0.000125, 0.0, 1.0);
	highp vec3 hp_extinction = vRM * densityH + (1.0 - densityH);
    
    uv0Var = vertex_uv0;
    extinction = hp_extinction * sunColorIntensity.rgb;
    inscatter = min(hp_inscatter, 1.75);
    density = vec4(densityH * -6.0 + 6.0, densityH * 5.0 - 5.0, -0.385 * densityH, densityH);
	gl_Position = pvwMat * vertex_pos;
    gl_Position.z = gl_Position.w*0.99999;
}
