attribute	highp	vec4	vertex_pos;
attribute	mediump vec3	vertex_normal;
attribute	highp   vec2	vertex_uv0;

uniform		highp	mat4	pvwMat;
uniform		highp	mat3	worldMat;
uniform		highp	vec3	eyePosMS;
uniform		lowp	vec3	lightDirMS;
uniform		lowp	vec3	lightAmbient;
uniform		lowp	vec3	lightDiffuse;
uniform		lowp	vec3	lightSpecular;
uniform		mediump float	materialShininess;

varying		highp   vec2	uv0Var;
varying		highp   vec2	uv1Var;
varying		lowp	vec3	ambientDiffuseVar;
varying		lowp	vec3	specularVar;
varying		lowp	float	reflDownAmmount;

const highp float OneOverPi = 0.318309886183791;

void main()
{
	highp vec3 eyeDirMS	 = normalize(eyePosMS - vertex_pos.xyz);
	highp vec3 halfVecMS = normalize(lightDirMS + eyeDirMS);
	
	uv0Var				= vertex_uv0;
	ambientDiffuseVar	= max(0.3, dot(vertex_normal, lightDirMS)) * lightDiffuse + lightAmbient;
	specularVar			= pow(max(0.0, dot(vertex_normal, halfVecMS)), materialShininess) * lightSpecular;

	highp vec3 reflDirMS = worldMat * reflect(-eyeDirMS, vertex_normal);
	highp float d = clamp(acos(reflDirMS.y) * OneOverPi * 1.3, 0.0, 1.0);
	highp float a = atan(-reflDirMS.x, -reflDirMS.z);

	uv1Var.x = cos(a) * d * 0.5 + 0.5;
	uv1Var.y = sin(a) * d * 0.5 + 0.5;
	
	reflDownAmmount = clamp(100.0 * (1.0 - clamp((0.6 + reflDirMS.y) / 0.4 + 1.0, 0.0, 1.0)), 0.0, 1.0);
	
	gl_Position = pvwMat * vertex_pos;
}
