attribute	vec4	vertex_pos;
attribute	vec2	vertex_uv0;

uniform		mat4	pvwMat;
uniform		vec2	detailRepeat;

uniform vec3    v3EyePosMS;
uniform float   fDepthCamera1;
uniform float   fLightScale;

uniform float   fIntensity;
uniform vec3    v3InvWavelength4KrKm4Pi;
uniform vec3    v3InvWavelength4KrKmSunExp;

const float fPi             = 3.141592653589793;
const float fESun           = 20.0;
const float fKr             = 0.0025;
const float fKm             = 0.0015;
const int   nSamples        = 6;
const float fSamplesInv     = 1.0 / 6.0;
const float fInnerRadius    = 10.0;
const float fOuterRadius    = 10.25;
const float fScaleDepth     = 0.25;
const float fExp            = -2.0;
const float fKr4Pi                  = fKr*4.0*fPi;
const float fKm4Pi                  = fKm*4.0*fPi;
const float fKrESunExp              = fKr*fESun*fExp;
const float fKmESunExp              = fKm*fESun*fExp;
const float fScale                  = 1.0 / (fOuterRadius - fInnerRadius);
const float fScaleOverScaleDepth    = fScale / fScaleDepth;

varying lowp vec3 color0;
varying lowp vec3 color1;
varying vec2 uv0Var;
varying vec2 uv1Var;

float scale(float fCos)
{
	float x = 1.0 - fCos;
	return fScaleDepth * exp(x*(x*(x*(x*5.25-6.8)+3.83)+0.459)-0.00287);
}

void main(void)
{
	vec3 v3Pos = vertex_pos.xyz / 637100.0;
	vec3 v3Ray = v3Pos - v3EyePosMS;
	float fFar = length(v3Ray);
	v3Ray /= fFar;
    fFar *= fIntensity;
    
	float fCameraScale  = scale(max(0.0, -v3Ray.y));
	float fCameraOffset = fDepthCamera1 * fCameraScale;
	float fTemp         = fLightScale + fCameraScale;
    
	float fSampleLength = fFar * fSamplesInv;
	float fScaledLength = fSampleLength * fScale;
	float fSampleRay    = v3Ray.y * fSampleLength;
	float fSamplePoint  = fSampleRay * -0.5 - v3EyePosMS.y;
    
	vec3 v3FrontColor = vec3(0.0);
	vec3 v3Attenuate;
	for(int i=0; i<nSamples; i++)
	{
		float fDepth    = exp(fScaleOverScaleDepth * fSamplePoint);
		float fScatter  = fDepth * fTemp - fCameraOffset;
		v3Attenuate     = exp(-fScatter * v3InvWavelength4KrKm4Pi);
		v3FrontColor += v3Attenuate * (fDepth * fScaledLength);
		fSamplePoint -= fSampleRay;
	}
    
    vec4 scatter = exp(vec4(v3FrontColor * v3InvWavelength4KrKmSunExp, vertex_pos.y * -0.000125));
    scatter.w = min(1.0, 1.21 * scatter.w);
    color0 = scatter.rgb * -scatter.w + scatter.w;
	color1 = clamp(v3Attenuate * ((fFar * -2.0 + 1.0) * scatter.w + (1.0 - scatter.w)), 0.0, 1.0);

    uv0Var = vertex_uv0;
    uv1Var = (vertex_uv0 * 2.0 - 1.0) * detailRepeat;
	gl_Position = pvwMat * vertex_pos;
}
