attribute	highp	vec4	vertex_pos;
//attribute   highp   vec3    vertex_normal;
attribute	highp	vec2	vertex_uv0;

uniform		highp	mat4	pvwMat;
uniform		highp	vec3	eyePosMS;
uniform		highp	vec2	detailRepeat;

uniform		highp	vec4	sunColorIntensity;
uniform		highp	vec3	sunDir;

uniform		highp	vec3	vBetaDashRayleigh;
uniform		highp	vec3	vBetaDashMie;
uniform		highp	vec3	vBetaRayleighMie;
uniform		highp	vec3	vHG;

uniform		highp	float	fogStart;
uniform		highp	float	fogEnd;
const		highp	float	scatterMaxDist = 100000.0;

varying		highp	vec2	uv0Var;
varying		highp	vec2	uv1Var;

varying		mediump  vec3	extinction;
varying		mediump  vec3	inscatter;
varying		mediump  vec2	density;

void main()
{
	highp vec3  eyeDirMS = vertex_pos.xyz - eyePosMS;
	highp float eyeDist = clamp(length(eyeDirMS) * fogEnd - fogStart, 0.0, scatterMaxDist);
	eyeDirMS = normalize(eyeDirMS);
	
	highp float theta = max(dot(sunDir, eyeDirMS), 0.0);
	highp float phase = (theta * theta * theta + 1.0) * (theta * -0.2 + 1.0);
	highp float phaseThetaMie = vHG.z * -theta + vHG.y;
	phaseThetaMie = vHG.x * inversesqrt(phaseThetaMie) / phaseThetaMie;
	
	highp vec3 hp_inscatter = vBetaDashRayleigh * phase + vBetaDashMie * phaseThetaMie;
    hp_inscatter *= sunColorIntensity.rgb * sunColorIntensity.w;
    
    highp vec4 vRMD = exp(vec4(vBetaRayleighMie * -eyeDist, vertex_pos.y * -0.000125));
    hp_inscatter = hp_inscatter * -vRMD.xyz + hp_inscatter;
    
	highp float densityH = min(1.0, 1.21 * vRMD.w);
    //highp float diffuse = max(0.0, dot(vertex_normal, sunDir)) * 0.5 + 0.5;
    //highp float densityH2 = densityH * diffuse;
	//highp vec3 hp_extinction = vRMD.xyz * densityH2 + (diffuse - densityH2);
    highp vec3 hp_extinction = vRMD.xyz * densityH + (1.0 - densityH);
	
    uv0Var = vertex_uv0;
    uv1Var = (vertex_uv0 * 2.0 - 1.0) * detailRepeat;
    extinction = (sunColorIntensity.rgb * 2.0) * hp_extinction;
    inscatter = min(hp_inscatter, 1.75);
    density = vec2(-0.385 * densityH, densityH);
	gl_Position = pvwMat * vertex_pos;
}