attribute	highp	vec4	vertex_pos;
attribute	highp   vec2	vertex_uv0;

uniform		highp	mat4	pvwMat;
uniform		highp	vec3	eyePosMS;

uniform		highp	vec4	sunColorIntensity;
uniform		highp   vec3	lightDirMS;
uniform		highp	float	vertex_yOffset;

uniform		highp	vec3	vBetaDashRayleigh;
uniform		highp	vec3	vBetaDashMie;
uniform		highp	vec3	vBetaRayleighMie;
uniform		highp	vec3	vHG;

uniform		highp	float	fogStart;
uniform		highp	float	fogEnd;
const		highp	float	scatterMaxDist = 100000.0;

varying		highp	vec2	uv0Var;
varying		lowp    vec3	extinction;
varying		lowp    vec3	inscatter;

void main()
{
    highp vec3  eyeDirMS = vertex_pos.xyz - eyePosMS;
	highp float eyeDist = clamp(length(eyeDirMS) * fogEnd - fogStart, 0.0, scatterMaxDist);
	eyeDirMS = normalize(eyeDirMS);

	highp float theta = max(dot(lightDirMS, eyeDirMS), 0.0);
	highp float phase = (theta * theta * theta + 1.0) * (theta * -0.2 + 1.0);
	highp float phaseThetaMie = vHG.z * -theta + vHG.y;
	phaseThetaMie = vHG.x * inversesqrt(phaseThetaMie) / phaseThetaMie;
	
	highp vec3 hp_inscatter = vBetaDashRayleigh * phase + vBetaDashMie * phaseThetaMie;
    hp_inscatter *= sunColorIntensity.rgb * sunColorIntensity.w;
    
    highp vec4 vRMD = exp(vec4(vBetaRayleighMie * -eyeDist, (vertex_pos.y + vertex_yOffset) * -0.000125));
    hp_inscatter = hp_inscatter * -vRMD.xyz + hp_inscatter;
    hp_inscatter = min(hp_inscatter, 1.75);
    hp_inscatter = hp_inscatter * hp_inscatter * (hp_inscatter * -0.385 + 1.0);
    
    highp float densityH = min(1.0, 1.21 * vRMD.w);
    hp_inscatter *= densityH;
	highp vec3 hp_extinction = vRMD.xyz * densityH + (1.0 - densityH);
    hp_extinction *= sunColorIntensity.rgb;
    
    uv0Var = vertex_uv0;
    extinction = min(hp_extinction, 1.995);
    inscatter = min(hp_inscatter, 1.995 - hp_extinction);
	gl_Position = pvwMat * vertex_pos;
}
