attribute	highp	vec4	vertex_pos;

uniform		highp	mat4	pvwMat;
uniform		highp	vec3	eyePos;
uniform		highp	vec4	sunColorIntensity;
uniform		highp	vec3	sunDir;

uniform		highp	vec3	vBetaDashRayleigh;
uniform		highp	vec3	vBetaDashMie;
uniform		highp	vec3	vBetaRayleighMie;
uniform		highp	vec3	vHG;

varying		highp	vec3	screenInfo;
varying		highp	vec2	alphaVar;
varying		lowp	vec3	extinction;
varying		lowp	vec3	inscatter;

uniform		highp	float	fogStart;
uniform		highp	float	fogEnd;
const		highp	float	scatterMaxDist = 100000.0;

const		highp	float	terrainSize	= 60000.0;
const		highp	float	terrainHSize= terrainSize * 0.5;

void main()
{
	highp vec3  eyeDirMS = vec3(vertex_pos.x, -eyePos.y, vertex_pos.z);
	highp float eyeDist = clamp(length(eyeDirMS) * fogEnd - fogStart, 0.0, scatterMaxDist);
	eyeDirMS = normalize(eyeDirMS);
	
	highp float theta = max(dot(sunDir, eyeDirMS), 0.0);
	highp float phase = (theta * theta * theta + 1.0) * (theta * -0.2 + 1.0);
	highp float phaseThetaMie = vHG.z * -theta + vHG.y;
	phaseThetaMie = vHG.x * inversesqrt(phaseThetaMie) / phaseThetaMie;
	
	highp vec3 hp_inscatter = vBetaDashRayleigh * phase + vBetaDashMie * phaseThetaMie;
    hp_inscatter *= sunColorIntensity.rgb * sunColorIntensity.w;
    
    highp vec3 hp_extinction = exp(vBetaRayleighMie * -eyeDist);
    hp_inscatter = hp_inscatter * -hp_extinction + hp_inscatter;
	hp_inscatter = min(hp_inscatter, 1.75);
    hp_inscatter = hp_inscatter * hp_inscatter * (hp_inscatter * -0.385 + 1.0);
    
	gl_Position = pvwMat * vertex_pos;
	screenInfo	= vec3((gl_Position.xy + gl_Position.w) * 0.5, gl_Position.w);
	alphaVar = (vertex_pos.xz + eyePos.xz + terrainHSize) / terrainSize;
    extinction = hp_extinction * sunColorIntensity.rgb;
    inscatter = hp_inscatter;
}
