attribute	vec4	vertex_pos;

uniform		mat4	pvwMat;

uniform     vec4    screenOffset;
uniform     vec3    eyePos;

uniform vec3    v3EyePosMS;
uniform float   fDepthCamera1;
uniform float   fLightScale;

uniform float   fIntensity;
uniform vec3    v3InvWavelength4KrKm4Pi;
uniform vec3    v3InvWavelength4KrKmSunExp;

const float fPi             = 3.141592653589793;
const float fESun           = 20.0;
const float fKr             = 0.0025;
const float fKm             = 0.0015;
const int   nSamples        = 6;
const float fSamplesInv     = 1.0 / 6.0;
const float fInnerRadius    = 10.0;
const float fOuterRadius    = 10.25;
const float fScaleDepth     = 0.25;
const float fExp            = -2.0;
const float fKr4Pi                  = fKr*4.0*fPi;
const float fKm4Pi                  = fKm*4.0*fPi;
const float fKrESunExp              = fKr*fESun*fExp;
const float fKmESunExp              = fKm*fESun*fExp;
const float fScale                  = 1.0 / (fOuterRadius - fInnerRadius);
const float fScaleOverScaleDepth    = fScale / fScaleDepth;

const		highp	float	terrainSize	= 60000.0;
const		highp	float	terrainHSize= terrainSize * 0.5;

varying		highp	vec4	screenInfo;
varying		highp	vec2	alphaVar;
varying		mediump	vec3	color0;
varying		mediump	vec3	color1;

float scale(float fCos)
{
	float x = 1.0 - fCos;
	return fScaleDepth * exp(x*(x*(x*(x*5.25-6.8)+3.83)+0.459)-0.00287);
}

void main(void)
{
	vec3 v3Pos = vertex_pos.xyz / 637100.0;
	vec3 v3Ray = v3Pos - v3EyePosMS;
	float fFar = length(v3Ray);
	v3Ray /= fFar;
    fFar *= fIntensity;
    
	float fCameraScale  = scale(-v3Ray.y);
	float fCameraOffset = fDepthCamera1 * fCameraScale;
	float fTemp         = fLightScale + fCameraScale;
    
	float fSampleLength = fFar * fSamplesInv;
	float fScaledLength = fSampleLength * fScale;
	float fSampleRay    = v3Ray.y * fSampleLength;
	float fSamplePoint  = fSampleRay * -0.5 - v3EyePosMS.y;
    
	vec3 v3FrontColor = vec3(0.0);
	vec3 v3Attenuate;
	for(int i=0; i<nSamples; i++)
	{
		float fDepth    = exp(fScaleOverScaleDepth * fSamplePoint);
		float fScatter  = fDepth * fTemp - fCameraOffset;
		v3Attenuate     = exp(-fScatter * v3InvWavelength4KrKm4Pi);
		v3FrontColor += v3Attenuate * (fDepth * fScaledLength);
		fSamplePoint -= fSampleRay;
	}
    
	color0 = 1.0 - exp(v3FrontColor * v3InvWavelength4KrKmSunExp);
	color1 = clamp(v3Attenuate * (fFar * -2.0 + 1.0), 0.0, 1.0);
	gl_Position = pvwMat * vertex_pos;
    
    float hlen = length(vertex_pos.xz);
    screenInfo	= vec4((gl_Position.xy + gl_Position.w) * 0.5 * screenOffset.zw, gl_Position.w,
					    clamp(eyePos.y * 0.000333 - 0.333333, 0.0, 1.0) * (hlen * -0.0001 + 10.0) + max(50000.0 - hlen, 0.0));
	alphaVar = (vertex_pos.xz + eyePos.xz + terrainHSize) / terrainSize;
}