uniform		sampler2D	tex0; //bump0 texture
uniform		sampler2D	tex1; //bump1 texture
uniform     sampler2D   tex2; //color texture
uniform     sampler2D   tex3; //reflection texture
uniform     sampler2D   tex4; //refraction texture
uniform     sampler2D   tex5; //foam texture

uniform		mediump	vec3	sunDir;
uniform		mediump	vec3	normalBump;
uniform     mediump vec4    foamAnim;
uniform     highp   vec4    screenOffset;

varying		highp	vec4	tex0Var;
varying		highp	vec4	tex1Var;
varying		highp	vec3	eyeDirMSVar;
varying     highp   vec3    screenInfo;
varying		highp	vec4    colorVar;
varying		mediump	vec3    extinction;
varying		mediump	vec3    inscatter;
varying		mediump	vec3	inscatter0;

const mediump float	specPow		= 48.0;
const mediump float kReflBump   = 0.04;
const mediump float kRefrBump   = 0.016;

void main()
{
	mediump	vec3  normal0	= normalize(texture2D(tex0, tex0Var.xy).xyz + texture2D(tex1, tex0Var.zw).xyz - 1.0);
    mediump vec3  normal1   = normalize(texture2D(tex1, tex1Var.xy).xyz + texture2D(tex1, tex1Var.zw).xyz - 1.0);
    mediump vec3  normal    = normalize(normal0 * 2.0 + normal1);
    mediump vec3  color     = texture2D(tex2, colorVar.xy).rgb;
    mediump float foam      = texture2D(tex5, colorVar.zw).r;
    
    highp vec2 scr          = screenInfo.xy / screenInfo.z + screenOffset.xy;
	mediump vec3  refl      = texture2D(tex3, normal.xz * kReflBump + scr).rgb;
    mediump vec4  refr      = texture2D(tex4, normal.xz * kRefrBump + scr);
    
    mediump	vec3  eye		= normalize(eyeDirMSVar);
	mediump vec3  halfVec	= normalize(eye + sunDir);
    mediump vec2  sPfR      = vec2(dot(normal, halfVec), dot(normal * normalBump, eye));
    
    sPfR.x = sPfR.x * specPow - specPow;
    sPfR   = exp(sPfR);
    sPfR.x = normal.x * normal.x + sPfR.x;
    sPfR.x *= 1.0 - refr.a;
    color += refl * sPfR.y + sPfR.x;
    
    mediump float f0 = clamp((abs(refr.a - 1.0) * -8.0 + 1.0) * normal0.y, 0.0, 1.0);
    mediump float f1 = clamp((abs(refr.a - foamAnim.x) * -10.0 + 1.0) * normal0.y, 0.0, 1.0) * foamAnim.y;
    
    color = mix(color, refr.rgb, refr.a * 0.5) + max(200.0 * sPfR.x - 180.0, 0.0);
    color = foam * (f0 * f0 + f1 * f1) + color;
    
    mediump vec3 scatter = (inscatter * inscatter) * inscatter0;
    
	gl_FragColor = vec4(color * extinction + scatter, 0.25);
}
