attribute	highp	vec4	vertex_pos;

uniform		highp	mat4	pvwMat;
uniform		highp	vec3	eyePos;
uniform		highp	vec4	sunColorIntensity;
uniform		highp	vec3	sunDir;

uniform		highp	vec3	vBetaDashRayleigh;
uniform		highp	vec3	vBetaDashMie;
uniform		highp	vec3	vBetaRayleighMie;
uniform		highp	vec3	vHG;
uniform		highp	vec3	bumpOffset;
uniform     highp   vec4    screenOffset;

varying		highp	vec4	tex0Var;
varying		highp	vec4	tex1Var;
varying		highp	vec3	eyeDirMSVar;
varying     highp   vec3    screenInfo;
varying		highp	vec4    colorVar;
varying		mediump	vec3    extinction;
varying		mediump	vec3    inscatter;
varying		mediump	vec3	inscatter0;

uniform		highp	float	fogStart;
uniform		highp	float	fogEnd;
const		highp	float	scatterMaxDist = 100000.0;

const		highp	float	zFar		= 50000.0 * 1.1;
const		highp	float	bumpScale	= 8.0 / zFar;

const		highp	float	terrainSize	= 60000.0;
const		highp	float	terrainHSize= terrainSize * 0.5;

void main()
{
	highp vec2 bump	= vertex_pos.zx * bumpScale + bumpOffset.xy;
	tex0Var.xy = bump + bumpOffset.z;
	tex0Var.zw = bump*4.0 - bumpOffset.z;
    tex1Var.xy = bump*8.0 + vec2(-bumpOffset.z, +bumpOffset.z);
    tex1Var.zw = bump*12.0 + vec2(+bumpOffset.z, -bumpOffset.z);
    colorVar.xy = (vertex_pos.xz + eyePos.xz + terrainHSize) / terrainSize;
    colorVar.zw = bump * 48.0;
    
    eyeDirMSVar.xz = -vertex_pos.xz;
	eyeDirMSVar.y = eyePos.y;
    
	highp vec3  eyeDirMS = vec3(vertex_pos.x, -eyePos.y, vertex_pos.z);
	highp float eyeDist = clamp(length(eyeDirMS) * fogEnd - fogStart, 0.0, scatterMaxDist);
	eyeDirMS = normalize(eyeDirMS);
	
	highp float theta = max(dot(sunDir, eyeDirMS), 0.0);
	highp float phase = (theta * theta * theta + 1.0) * (theta * -0.2 + 1.0);
	highp float phaseThetaMie = vHG.z * -theta + vHG.y;
	phaseThetaMie = vHG.x * inversesqrt(phaseThetaMie) / phaseThetaMie;
	
	highp vec3 hp_inscatter = vBetaDashRayleigh * phase + vBetaDashMie * phaseThetaMie;
    hp_inscatter *= sunColorIntensity.rgb * sunColorIntensity.w;
    
    highp vec3 hp_extinction = exp(vBetaRayleighMie * -eyeDist);
    hp_inscatter = hp_inscatter * -hp_extinction + hp_inscatter;
    
	gl_Position = pvwMat * vertex_pos;
	screenInfo	= vec3((gl_Position.xy + gl_Position.w) * 0.5 * screenOffset.zw, gl_Position.w);
    extinction = hp_extinction * sunColorIntensity.rgb;
    inscatter = min(hp_inscatter, 1.75);
    inscatter0 = inscatter * -0.385 + 1.0;
}